/// <reference types="node" />
import { RedisClientOptions, RedisClientType } from 'redis';
import '@redis/client';
import '@redis/bloom';
import '@redis/graph';
import '@redis/json';
import '@redis/search';
import '@redis/time-series';
import { Cache, Store } from 'cache-manager';
declare type CachingConfig = {
    ttl?: number | ((value: unknown) => number);
};
declare type CB<T> = (err: NodeJS.ErrnoException | null, result: T | null) => void;
declare type CBSet = (err: NodeJS.ErrnoException | null) => void;
export interface CacheManagerOptions {
    ttl?: number;
    isCacheableValue?: (value: unknown) => boolean;
}
export interface RedisCache extends Cache {
    store: RedisStore;
    set: RedisStore['set'];
    get: RedisStore['get'];
    del: RedisStore['del'];
    reset: RedisStore['reset'];
}
export interface RedisStore extends Store {
    name: 'redis';
    isCacheableValue: (value: unknown) => boolean;
    get getClient(): RedisClientType;
    set<T>(key: string, value: T, options: CachingConfig, callback: CBSet): void;
    set<T>(key: string, value: T, ttl: number, callback: CBSet): void;
    set<T>(key: string, value: T, options?: CachingConfig): Promise<void>;
    set<T>(key: string, value: T, ttl: number): Promise<void>;
    get<T>(key: string, opt: null, callback: CB<T>): void;
    get<T>(key: string): Promise<T | undefined>;
    del(key: string, callback: CBSet): void;
    del(key: string[], callback: CBSet): void;
    del(key: string): Promise<void>;
    del(key: string[]): Promise<void>;
    reset(cb: () => void): void;
    reset(): Promise<void>;
    mset(args: [string, unknown][], ttl: number | undefined, cb: CBSet): void;
    mset(args: [string, unknown][], ttl?: number): Promise<void>;
    mget(...args: [...string[], CB<unknown[]>]): void;
    mget(...args: string[]): Promise<unknown[]>;
    keys(pattern: string | undefined, cb: CB<string[]>): void;
    keys(pattern?: string): Promise<string[]>;
    ttl(key: string, cb: CB<number>): void;
    ttl(key: string): Promise<number>;
}
export declare function redisStore(options?: RedisClientOptions & CacheManagerOptions): Promise<RedisStore>;
export {};
