"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.redisStore = void 0;
const node_util_1 = require("node:util");
const redis_1 = require("redis");
require("@redis/client");
require("@redis/bloom");
require("@redis/graph");
require("@redis/json");
require("@redis/search");
require("@redis/time-series");
const getVal = (value) => JSON.stringify(value) || '"undefined"';
function redisStore(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const isCacheableValue = (options === null || options === void 0 ? void 0 : options.isCacheableValue) ||
            ((value) => value !== undefined && value !== null);
        const redisCache = (0, redis_1.createClient)(options);
        yield redisCache.connect();
        const get = (key) => __awaiter(this, void 0, void 0, function* () {
            const val = yield redisCache.get(key);
            if (val)
                return JSON.parse(val);
            else
                return null;
        });
        const set = (key, value, ttl) => __awaiter(this, void 0, void 0, function* () {
            if (!isCacheableValue(value))
                throw new Error(`"${value}" is not a cacheable value`);
            if (typeof ttl === 'object') {
                if (typeof ttl.ttl === 'function')
                    ttl = ttl.ttl(value);
                else
                    ttl = ttl.ttl;
            }
            if (ttl)
                yield redisCache.setEx(key, ttl, getVal(value));
            else
                yield redisCache.set(key, getVal(value));
        });
        const reset = () => __awaiter(this, void 0, void 0, function* () {
            yield redisCache.flushDb();
        });
        const mset = (args, ttl) => __awaiter(this, void 0, void 0, function* () {
            if (!ttl)
                ttl = options === null || options === void 0 ? void 0 : options.ttl;
            if (ttl) {
                const multi = redisCache.multi();
                for (const [key, value] of args) {
                    if (!isCacheableValue(value))
                        throw new Error(`"${getVal(value)}" is not a cacheable value`);
                    multi.setEx(key, ttl, getVal(value));
                }
                yield multi.exec();
            }
            else
                yield redisCache.mSet(args.map(([key, value]) => {
                    if (!isCacheableValue(value))
                        throw new Error(`"${getVal(value)}" is not a cacheable value`);
                    return [key, getVal(value)];
                }));
        });
        const mget = (...args) => redisCache
            .mGet(args)
            .then((x) => x.map((x) => (x === null ? null : JSON.parse(x))));
        return {
            name: 'redis',
            isCacheableValue,
            get getClient() {
                return redisCache;
            },
            set(key, value, ttl, cb) {
                if (typeof cb === 'function')
                    (0, node_util_1.callbackify)((set))(key, value, ttl, cb);
                else
                    return set(key, value, ttl);
            },
            get(key, opt, cb) {
                if (typeof cb === 'function')
                    (0, node_util_1.callbackify)((get))(key, cb);
                else
                    return get(key);
            },
            mset(args, ttl, cb) {
                if (typeof cb === 'function')
                    (0, node_util_1.callbackify)(mset)(args, ttl, cb);
                else
                    return mset(args, ttl);
            },
            mget(...args) {
                if (typeof args.at(-1) === 'function') {
                    const cb = args.pop();
                    (0, node_util_1.callbackify)(() => mget(...args))(cb);
                }
                else
                    return mget(...args);
            },
            del(args, cb) {
                const fn = () => __awaiter(this, void 0, void 0, function* () {
                    yield redisCache.del(args);
                });
                if (typeof cb === 'function')
                    (0, node_util_1.callbackify)(fn)(cb);
                else
                    return fn();
            },
            reset(cb) {
                if (typeof cb === 'function')
                    (0, node_util_1.callbackify)(reset)(cb);
                else
                    return reset();
            },
            keys(pattern = '*', cb) {
                if (typeof cb === 'function')
                    (0, node_util_1.callbackify)(() => redisCache.keys(pattern))(cb);
                else
                    return redisCache.keys(pattern);
            },
            ttl: (key, cb) => {
                if (typeof cb === 'function')
                    (0, node_util_1.callbackify)(() => redisCache.ttl(key))(cb);
                else
                    return redisCache.ttl(key);
            },
        };
    });
}
exports.redisStore = redisStore;
